@@ -2,7 +2,7 @@
 
     Various definitions for PGPsendmail.
 
-    Copyright (C) 1994-1997  Richard Gooch
+    Copyright (C) 1994-1998  Richard Gooch
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -69,8 +69,10 @@ typedef struct
     flag scan_to;
     flag scan_cc;
     flag scan_bcc;
-    flag scan_from;
+    flag scan_from_claimed;
+    flag scan_from_relay;
     flag scan_sender;
+    flag scan_reply_to;
     char *linebuf;
     unsigned int linebuf_size;
     unsigned int line_length;
