@@ -2,7 +2,7 @@
 
     Source file for option processing for  PGPsendmail  (wrapper to sendmail).
 
-    Copyright (C) 1994-1997  Richard Gooch
+    Copyright (C) 1994-1998  Richard Gooch
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -57,9 +57,11 @@
     Updated by      Richard Gooch   5-OCT-1997: Added support for "ALIAS"
   directive.
 
-    Last updated by Richard Gooch   9-OCT-1997: Made use of new function
+    Updated by      Richard Gooch   9-OCT-1997: Made use of new function
   <st_expr_expand>.
 
+    Last updated by Richard Gooch   23-JAN-1998: Added #include <errno.h>
+
 
 */
 #include <stdio.h>
@@ -69,7 +71,8 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <regex.h>
-#include "pgpsendmail.h"
+#include <errno.h>
+#include <pgpsendmail.h>
 
 
 /*  Private functions  */
