@@ -2,7 +2,7 @@
 
     Source file for  PGPdaemon  (automatic signer/encryptor for PGP).
 
-    Copyright (C) 1994-1997  Richard Gooch
+    Copyright (C) 1994-1999  Richard Gooch
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -157,14 +157,20 @@
     Updated by      Richard Gooch   8-OCT-1997: Supported $env as USERID
   configuration option.
 
-    Last updated by Richard Gooch   9-OCT-1997: Made use of new function
+    Updated by      Richard Gooch   9-OCT-1997: Made use of new function
   <st_expr_expand>.
 
     Updated by      Richard Gooch   21-OCT-1997: Conditionally include paths.h
 
-    Last updated by Richard Gooch   27-OCT-1997: Fixed bug if no logfile
+    Updated by      Richard Gooch   27-OCT-1997: Fixed bug if no logfile
   specified in configuration file.
 
+    Updated by      Richard Gooch   10-JUL-1998: Removed definitions of system
+  errlist array.
+
+    Last updated by Richard Gooch   13-FEB-1999: Added regular inversion of
+  passphrase
+
 
 */
 #include <stdio.h>
@@ -194,7 +200,6 @@
 #define MIN_INTERVAL 1
 #define DEFAULT_INTERVAL 15
 #define MAX_INTERVAL 60
-#define XOR_CHAR 0x1a
 
 STATIC_FUNCTION (flag service_connect,
 		 (CONST char *inpipe_filename, CONST char *outpipe_filename) );
@@ -210,6 +215,7 @@ STATIC_FUNCTION (void read_config, (char *pgppath, flag *mailwait,
 STATIC_FUNCTION (flag write_secring, () );
 STATIC_FUNCTION (flag scrub_secring, () );
 STATIC_FUNCTION (int get_rand, () );
+STATIC_FUNCTION (void invert_passphrase, () );
 
 
 /*  Private data  */
@@ -231,6 +237,7 @@ static char outgoing_spool[STRING_LENGTH];
 static char my_userid[STRING_LENGTH];
 static char logfile[STRING_LENGTH];
 static char hostname[STRING_LENGTH];
+static char xor_char = 0x1a;
 
 
 /*  Code for  myexit  must come first.  */
@@ -301,7 +308,6 @@ char *envp[];
     extern char hostname[STRING_LENGTH];
     static char usage_string[] =
     "Usage:\tpgpdaemon [-mailwait] [-detach] [-pgppath <path>]";
-    extern char *sys_errlist[];
 
     if (close (0) != 0)
     {
@@ -532,12 +538,7 @@ char *envp[];
     }
     while (keep_going)
     {
-	/*  Keep passphrase in RAM (hopefully out of swap) by touching it  */
-	for (count = 0, tmp = 0; count < STRING_LENGTH; ++count)
-	{
-	    tmp += passphrase[count];
-	}
-	tmp = 0;
+	invert_passphrase ();
 	sleep (1);
 	if ( login_session && (getppid () != ppid) )
 	{
@@ -613,7 +614,6 @@ static flag service_connect (CONST char *inpipe_filename,
     extern SECRING_SIZE_TYPE secring_bytes;
     extern char passphrase[STRING_LENGTH];
     extern char *secring_buf;
-    extern char *sys_errlist[];
 
     /*  Connect to output pipe  */
     for (count = 0; (count < 2) && (out_fd < 0); ++count)
@@ -689,7 +689,7 @@ static flag service_connect (CONST char *inpipe_filename,
 	passphrase[len++] = '\n';
 	passphrase[len] = '\0';
 	/*  Simple encryption of passphrase  */
-	for (count = 0; count < len; ++count) passphrase[count] ^= XOR_CHAR;
+	for (count = 0; count < len; ++count) passphrase[count] ^= xor_char;
 	if (secring_bytes > 0)
 	{
 	    m_clear (secring_buf, secring_bytes);
@@ -858,12 +858,11 @@ static flag process_message (int in_fd, int out_fd, flag sign, flag ascii)
     int pgp_err_fd = ERROR_FD;
     int child;
     int status;
-    int len, count;
+    int len, count, bytes_written;
     char tmp_passphrase[STRING_LENGTH];
     char *argv[7];
     extern char my_userid[STRING_LENGTH];
     extern char passphrase[STRING_LENGTH];
-    extern char *sys_errlist[];
 
     /*  Setup arguments to PGP  */
     if (sign)
@@ -905,17 +904,17 @@ static flag process_message (int in_fd, int out_fd, flag sign, flag ascii)
     len = strlen (passphrase);
     /*  Simple decryption of passphrase  */
     for (count = 0; count < len; ++count)
-	tmp_passphrase[count] = passphrase[count] ^ XOR_CHAR;
-    if (write (pgp_in_fd, tmp_passphrase, len) < len)
+	tmp_passphrase[count] = passphrase[count] ^ xor_char;
+    bytes_written = write (pgp_in_fd, tmp_passphrase, len);
+    m_clear (tmp_passphrase, len);
+    if (bytes_written < len)
     {
-	m_clear (tmp_passphrase, len);
 	fprintf (stderr, "Error writing passphrase\t%s\n", ERRSTRING);
 	close (pgp_in_fd);
 	waitpid (child, &status, 0);
 	if (remove_secring) scrub_secring ();
 	return (FALSE);
     }
-    m_clear (tmp_passphrase, len);
     /*  Copy input message to PGP  */
     if ( !copy_data (pgp_in_fd, in_fd, ascii) )
     {
@@ -979,7 +978,6 @@ flag lock;
     char *ptr;
     char lock_name[STRING_LENGTH];
     char mlk_name[STRING_LENGTH];
-    extern char *sys_errlist[];
 
     strcpy (lock_name, filename);
     strcat (lock_name, ".lock");
@@ -1055,7 +1053,6 @@ flag no_filelocking;
 #ifdef F_SETLK
     struct flock fl;
 #endif
-    extern char *sys_errlist[];
 
     if (no_filelocking) return (TRUE);
 #undef LOCKING_WORKS
@@ -1127,7 +1124,6 @@ static flag check_and_process_mail ()
     extern char incoming_spool[STRING_LENGTH];
     extern char outgoing_spool[STRING_LENGTH];
     extern char passphrase[STRING_LENGTH];
-    extern char *sys_errlist[];
 
     if (stat (incoming_spool, &statbuf) != 0)
     {
@@ -1343,7 +1339,7 @@ static flag check_and_process_mail ()
 	    len = strlen (passphrase);
 	    /*  Simple decryption of passphrase  */
 	    for (count = 0; count < len; ++count)
-		tmp_passphrase[count] = passphrase[count] ^ XOR_CHAR;
+		tmp_passphrase[count] = passphrase[count] ^ xor_char;
 	    if (write (in_fd, tmp_passphrase, len) < len)
 	    {
 		m_clear (tmp_passphrase, len);
@@ -1536,7 +1532,6 @@ flag check_mail;
     extern char incoming_spool[STRING_LENGTH];
     extern char outgoing_spool[STRING_LENGTH];
     extern char logfile[STRING_LENGTH];
-    extern char *sys_errlist[];
 
     sprintf (config_filename, "%s/PGPdaemon.config", pgppath);
     if ( ( fp = fopen (config_filename, "r") ) == NULL )
@@ -1682,7 +1677,6 @@ static flag write_secring ()
     extern SECRING_SIZE_TYPE secring_bytes;
     extern char *secring_buf;
     extern char *pgppath;
-    extern char *sys_errlist[];
 
     sprintf (filename, "%s/secring.pgp", pgppath);
     if ( ( fd = open (filename, O_WRONLY | O_CREAT | O_EXCL,
@@ -1735,5 +1729,28 @@ static int get_rand ()
 	tv.tv_sec ^= tv.tv_usec;
 	srand (tv.tv_sec);
     }
-    return ( rand () );
+    return rand ();
 }   /*  End Function get_rand  */
+
+static void invert_passphrase ()
+/*  [SUMMARY] Invert the passphrase and XOR character.
+    [PURPOSE] This function will invert the passphrase and XOR character. If
+    called regularly, it ensures that the passphrase buffer contents do not
+    remain static with time. This is important because it is possible to read
+    the previous contents of a memory chip even after it has been turned off.
+    The longer a bit in memory is unchanged, the longer it takes for the
+    afterimage to fade beyond recovery. Regularly inverting the buffer contents
+    serves to defeat attempts at data recovery. This function also helps to
+    keep the passphrase out of swap space.
+    [RETURNS] Nothing.
+*/
+{
+    int count;
+    int length = strlen (passphrase);
+
+    for (count = 0; count < length; ++count)
+    {
+	passphrase[count] = ~passphrase[count];
+    }
+    xor_char = ~xor_char;
+}   /*  End Function invert_passphrase  */
