@@ -2,7 +2,7 @@
 
     Source file for  PGPsendmail  (wrapper to sendmail).
 
-    Copyright (C) 1994-1997  Richard Gooch
+    Copyright (C) 1994-1998  Richard Gooch
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -79,7 +79,7 @@
   outgoing messages (even those going out in plaintext).
 
     Updated by      Richard Gooch   21-JUL-1994: Added declarations in
-  transmit_plaintext  for  errno  and  sys_errlist  .
+  transmit_plaintext  for  errno  and  sys_ errlist  .
 
     Updated by      Richard Gooch   15-AUG-1994: Added  add_key  parameter to
   transmit_plaintext  .
@@ -123,7 +123,13 @@
     Updated by      Richard Gooch   9-OCT-1997: Restored configurability of
   real sendmail location (else mere mortals would be hosed).
 
-    Last updated by Richard Gooch   21-OCT-1997: Conditionally include paths.h
+    Updated by      Richard Gooch   21-OCT-1997: Conditionally include paths.h
+
+    Updated by      Richard Gooch   15-APR-1998: Support BSD mailer which
+  passes "send-mail" as argv[0].
+
+    Last updated by Richard Gooch   10-JUL-1998: Removed definitions of system
+  errlist array.
 
 
 */
@@ -208,7 +214,6 @@ char *envp[];
     extern FILE *debug_fp;
     extern char *pgppath;
     extern char *username;
-    extern char *sys_errlist[];
 
     if (argv[0] == NULL)
     {
@@ -246,7 +251,13 @@ char *envp[];
 	fprintf (stderr, "Someone didn't read the installation notes...\n");
 	exit (RV_UNDEF_ERROR);
     }
-    if (strcmp (ptr2, "sendmail") != 0)
+    if (strcmp (ptr2, "mailq") == 0)
+    {
+	execve (real_sendmail, argv, envp);
+	exit (1);
+    }
+    if ( (strcmp (ptr2, "sendmail") != 0) &&
+	 (strcmp (ptr2, "send-mail") != 0) )
     {
 	sleep (1);
 	execve (real_sendmail, argv, envp);
@@ -831,7 +842,6 @@ char *message;
     FILE *err_fp;
     char *argv[6];
     extern char *username;
-    extern char *sys_errlist[];
 
     /*  Make stdout go to  stderr  */
     fflush (stdout);
@@ -972,7 +982,6 @@ static void transmit_plaintext (flag sign, FILE *debug_fp, int sendmail_pid,
     int child_pid, status;
     int to_fd, from_fd;
     char buffer[STRING_LENGTH];
-    extern char *sys_errlist[];
 
     if (!sign)
     {
